@@ -10,6 +10,7 @@
 \\withMusicProperty
 \\with-url
 \\with-link
+\\with-dimensions-from
 \\with-dimensions
 \\with-color
 \\with
@@ -106,6 +107,7 @@
 \\tagGroup
 \\tag
 \\table-of-contents
+\\table
 \\tablatureFormat
 \\tabStaffLineLayoutFunction
 \\tabFullNotation
@@ -790,6 +792,7 @@
 \\B
 \\AncientRemoveEmptyStaffContext
 \\A
+staff
 spacing
 signature
 routine
